#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print \pmatrix{} maths for the three matrices of each tree, in 3x3 and 2x2
# forms.

use strict;
use List::Util 'max';

use FindBin;
use lib $FindBin::Bin;
use MyTriples ':all';

my $type = 'UAD';
if (@ARGV) {
  $type = shift @ARGV;
}
print "% generated by tools/$FindBin::Script $type\n";

# Called print_pmatrix_3x3($name, $m3x3, $m2x2,
#                          $name, ...)
# The matrices are Math::Matrix objects.
# Print a row of 3x3 with their names,
# then print a row of the 2x2.
#
sub print_pmatrix_3x3 {
  my @elems = @_;
  {
    # each 3x3
    my $join = '';
    foreach my $elem (@elems) {
      my ($name,$m33,$m22, $gpname33,$gpname22) = @$elem;
      my @row_wise = ($m33->[0][0], $m33->[0][1], $m33->[0][2],
                      $m33->[1][0], $m33->[1][1], $m33->[1][2],
                      $m33->[2][0], $m33->[2][1], $m33->[2][2]);
      my $indent = length($name) + 1;
      if (length($name) > 1) {
        $name = "\\mathit{$name}\n   ";
        $indent = 4;
      }
      print $join;
      printf "%s &= \\begin{pmatrix*}[r] %2d & %2d & %2d \\\\ %2d & %2d & %2d \\\\ %2d & %2d & %1d\n",
        $name, @row_wise;
      printf "%*s   \\end{pmatrix*}\n",
        $indent, '';
      printf "%% GP-Test  %s == [%d,%d,%d; %d,%d,%d; %d,%d,%d]\n",
        $gpname33, @row_wise;
      $join = "&\n";
    }
  }
  print "\\notag\n\\\\\n";

  {
    # each 2x2
    my $join = '';
    foreach my $elem (@elems) {
      my ($name,$m33,$m22, $gpname33,$gpname22) = @$elem;
      my $indent = length($name) + 1;
      if (length($name) > 1) {
        $name = "\\mathit{$name}\n   ";
        $indent = 4;
      }
      my @row_wise = ($m22->[0][0], $m22->[0][1],
                      $m22->[1][0], $m22->[1][1]);
      print $join;
      printf "%*s&= \\begin{pmatrix*}[r] %2d & %2d \\\\ %2d & %2d \\end{pmatrix*}\n",
        $indent, '', @row_wise;
      printf "%% GP-Test  %s == [%d,%d; %d,%d]\n",
        $gpname22, @row_wise;
      $join = "&\n";
    }
  }
}

if ($type eq 'UAD') {
  print_pmatrix_3x3(['U', U_3x3(), U_2x2(), 'u3','u'],
                    ['A', A_3x3(), A_2x2(), 'a3','a'],
                    ['D', D_3x3(), D_2x2(), 'd3','d']);
} elsif ($type eq 'FB') {
  print_pmatrix_3x3(['M1', M1_3x3(), M1_2x2(), 'm1_3','m1'],
                    ['M2', M2_3x3(), M2_2x2(), 'm2_3','m2'],
                    ['M3', M3_3x3(), M3_2x2(), 'm3_3','m3']);
} elsif ($type eq 'UMT') {
  print_pmatrix_3x3(['U',  U_3x3(),  U_2x2(),  'u3', 'u'],
                    ['M2', M2_3x3(), M2_2x2(), 'm2_3','m2'],
                    ['T',  T_3x3(),  T_2x2(),  't3', 't']);
} else {
  die "Unrecognised type: $type";
}

print "% end generated\n";
exit 0;
